#include "FIFO.h"

FIFO::FIFO(uint8_t *memory, uint16_t sizeAllocated) {	// {{{ use: uint8_t arr[80]; FIFO fifo(arr[0],80);
	mem = memory;
	size = sizeAllocated;
	used = 0;
	head = 0;
	tail = 0;
	total_count = 0;
}	// }}}
bool FIFO::notFull() {		// {{{ true if there is place for another fifo.push(val)
	return used < size;
}	// }}}
bool FIFO::notEmpty() {	// {{{ true if fifo.get() can return something
	return used;
}	// }}}
uint8_t FIFO::push(uint8_t val) {	// {{{ return val. If fifo is already full, then nothing is changed
	++total_count;
	if (used > size) return val;
	mem[head++] = val;
	if (head>=size) head=0;
	++ used;
	return val;
}	// }}}
uint8_t FIFO::get() {		// {{{ if notEmpty() then returns next value, else returns 0
	if (! used) return 0;
	uint8_t retval=mem[tail++];
	if (tail >= size) tail=0;
	--used;
	return retval;
}	// }}}
uint16_t FIFO::Total_Count() {	// {{{ returns total_count of tried pushes
	return total_count;
}	// }}}
		// ----
uint8_t FIFO::peek() {		// {{{ returns what get would return now, but does not change anything
	if (! used) return 0;
	return mem[tail];
}	// }}}
uint16_t FIFO::Free() {	// {{{ number of actually free positions
	return size - used;
}	// }}}
uint16_t FIFO::Used() {	// {{{ number of actually used positions
	return used;
}	// }}}
uint16_t FIFO::Total() {	// {{{ total size of FIFO = sizeAllocated from constructor
	return size;
}	// }}}
