#include "FIFO_text.h"

FIFO_text::FIFO_text(uint8_t *memory, uint16_t sizeAllocated): FIFO(memory,sizeAllocated) {	// {{{ use: uint8_t arr[80]; FIFO fifo(arr[0],80);
	clear();
}	// }}}
uint8_t FIFO_text::push(uint8_t val) {			// {{{ return val. If fifo is already full, then nothing is changed
	FIFO::push(val);
	rescan();
}	// }}}
uint8_t FIFO_text::get() {					// {{{ if notEmpty() then returns next value, else returns 0
	uint8_t retval=FIFO::get();
	for (uint8_t i=0; i<used;++i) mem[i]=mem[i+1];
	tail--;
	if (head==0) head=size;
	else head--;
	rescan();
	return retval;
}	// }}}
void FIFO_text::clear() {						// {{{ reset head, tail, and used to 0. Called in get, if the FIFO is Empty afterward
	head = tail = used = 0;
	firstBlank = firstEoln = Word = Eoln = false;
}	// }}}
static bool FIFO_text::isBlank(uint8_t c) {				// c is ' ', \t, \r or \n
	return (c == ' ' || c == '\t' || c == '\r' || c == '\n' || c == 0 );
}	// }}}
void FIFO_text::rescan() {						// {{{ sets firstBlank, Word, Eoln
	firstBlank = firstEoln = Word = Eoln = false;
	if (! used) return;
	firstBlank = isBlank(mem[tail]);
	firstEoln = (mem[tail] == '\r' || mem[tail] == '\n' || mem[tail] == 0);
	bool b = firstBlank, bb;
	for (uint8_t cnt = 0, i = tail; cnt < used; ++cnt) {
		uint8_t c = mem[i++];
		if (i>=size) i=0;
		if (c == '\r' || c == '\n' || c == 0) Eoln=true;
		if ((bb = isBlank(c))) if (!b) { Word = true; };
		b=bb;
	};
}	// }}}
