/* vim:		noexpandtab fileencoding=utf-8 nomodified nowrap textwidth=200 foldmethod=marker foldmarker={{{,}}} foldcolumn=4 ruler showcmd lcs=tab\:|- list: 
*/
#define VERZE "Expanduino_I v.0.0 (test Speed)"
#define SpeedInterval 1000
// Provide SpeedInterval ms long pulzes and pauses on Status.0
// prints, how many cycles did on reading/writing Status
// ignores all registers etc. (pause=Status.0==0 is longer for logging)

/*
	Expanduino_I card for 6809 (Omen Kilo) 115.200 Bd, ANSI color
	HW: Arduino Micro Pro + Expanduino_I; see http://8bit.gilhad.cz/6809/Expanduino/Expanduino_I.html
	TL;DR;	I/O card for 6809 with benefits
*/
// #define SERIAL_SPEED 9600
#define SERIAL_SPEED 115200
#include <Wire.h>
#include <SPI.h>
// I2C device found at address 0x57  ! AT24C32 EEPROM 32K (4096 x 8)
// I2C device found at address 0x68  ! RTC DS3231
#define AT24C32_I2C_ADDRESS 0x57
#define DS3231_I2C_ADDRESS 0x68

//
#include <stdint.h>
#include "FIFO.h"
#include "FIFO_text.h"
// {{{ fifo[8] 
#define BUFF_SIZE 80
uint8_t b0,buff[7][BUFF_SIZE];
FIFO fifo[8] = {
	FIFO(&b0,1),
	FIFO(buff[0],BUFF_SIZE),
	FIFO(buff[1],BUFF_SIZE),
	FIFO(buff[2],BUFF_SIZE),
	FIFO(buff[3],BUFF_SIZE),
	FIFO(buff[4],BUFF_SIZE),
	FIFO(buff[5],BUFF_SIZE),
	FIFO(buff[6],BUFF_SIZE)
	};
char cmd_line_buff[BUFF_SIZE+1];
FIFO_text cmd_line(cmd_line_buff, BUFF_SIZE);
// aliases
FIFO &R00 = fifo[0];	// Status Register, no usable FIFO
FIFO &W00 = fifo[1];	// val=W00.get()
FIFO &R01 = fifo[2];
FIFO &W01 = fifo[3];
FIFO &R02 = fifo[4];
FIFO &W02 = fifo[5];
FIFO &R03 = fifo[6];
FIFO &W03 = fifo[7];
//
FIFO *Rx[4] = { &R00, &R01, &R02, &R03};	// Rx[1]->push(123);
FIFO *Wx[4] = { &W00, &W01, &W02, &W03};
// }}}
// {{{ Pins
// I2C
#define P_I2C_SDA	2
#define P_I2C_CLK	3
// Status - reg0
#define P_SR_reg0	4
#define StatusOn	digitalWrite( P_SR_reg0, LOW)
#define StatusOff	digitalWrite( P_SR_reg0, HIGH)
// Free to use
#define P_D5	5
#define P_D6	6
#define P_D7	7
#define P_D8	8
#define P_D9	9
// SPI
#define P_Demux2	10
#define P_SerW		16
#define P_SerRxx	14
#define P_clock		15
#define P_Sel_Enable	18
#define P_Sel_0		19
#define P_Sel_1		20
#define P_Sel_2		21
// Slave Select must be OUTPUT
#define P_Ser_SS	17
// Arduino LEDs
#define P_RX		17
#define P_TX		30
// }}}
// {{{ SPI
#define RxOn		digitalWrite( P_RX, LOW )
#define RxOff		digitalWrite( P_RX, HIGH )
#define RxSet(On)	digitalWrite( P_RX, !On )

#define TxOn		digitalWrite( P_TX, LOW )
#define TxOff		digitalWrite( P_TX, HIGH )
#define TxSet(On)	digitalWrite( P_TX, !On )

void SelectSPI(bool Dem2, bool S2, bool S1, bool S0) {	// {{{ Select SPI and Enable it
	// prepare all
	digitalWrite( P_Demux2,		Dem2 );	// P_Demux2
	digitalWrite( P_SerW,		HIGH );	// P_SerW   will be driven anyway 
	// digitalWrite( P_SerRxx,	HIGH );	// P_SerRxx is INPUT
	digitalWrite( P_clock,		LOW  );	// P_clock  works on rising edge, so start LOW to not risk pulse
	digitalWrite( P_Sel_0,		S0 );	// P_Sel_0
	digitalWrite( P_Sel_1,		S1 );	// P_Sel_1
	digitalWrite( P_Sel_2,		S2 );	// P_Sel_2
	// digitalWrite( P_Ser_SS,	HIGH );	// P_Ser_SS ignored, must be OUTPUT
	//
	digitalWrite( P_Sel_Enable,	LOW );	// P_Sel_Enable - last to make it work
	// LEDs and such
	//
	uint8_t addr=0; 	// {{{ 
	if (Dem2) { addr |= (1 << 3); }
	if (S2)   { addr |= (1 << 2); }
	if (S1)   { addr |= (1 << 1); }
	if (S0)   { addr |= (1 << 0); }
	// }}}
	if (addr) {		// {{{ Do not show Status Register
		RxSet(S0);
		TxSet(!S0);
	} else { 
		RxOff;TxOff; 
	};	// }}}
}	// }}}
void SelectSPI(uint8_t addr) {	// {{{ Select SPI and Enable it easier way
	SelectSPI(bitRead(addr, 3), bitRead(addr, 2), bitRead(addr, 1), bitRead(addr, 0));
}	// }}}
void DeSelectSPI() {	// {{{ DeSelect SPI and Disable it
	digitalWrite( P_Sel_Enable,	HIGH );	// P_Sel_Enable - first to make it work
	// it was easy :)
	RxOff;TxOff; 
}	// }}}
uint8_t WriteSPI(uint8_t addr, uint8_t val) {	// {{{ Select SPI, writes byte and then DeSelect it
	SelectSPI(bitRead(addr, 3), bitRead(addr, 2), bitRead(addr, 1), bitRead(addr, 0));
	uint8_t retval = SPI.transfer(val);
	DeSelectSPI();
	return retval;
}	// }}}
uint8_t ReadSPI(uint8_t addr) {	// {{{ Select SPI, reads it and then DeSelect it
	SelectSPI(bitRead(addr, 3), bitRead(addr, 2), bitRead(addr, 1), bitRead(addr, 0));
	uint8_t retval = SPI.transfer(0);
	DeSelectSPI();
	return retval;
}	// }}}
// }}}
#define CheckSerial1	true
#define CheckSerial2	Serial
#define CheckSerial3	UDADDR & _BV(ADDEN)
#define CheckSerial4	USBSTA & _BV(VBUS)
#define CheckSerial5	Serial.availableForWrite()
#define CheckSerial CheckSerial1
//
void ManageRegisters() {	// {{{ Automagically transfer write registers to buffers and buffers to read registers, if possible - call in loop()
	uint8_t status = ReadSPI(0);	// Status Register
	for (uint8_t i = 1; i < 8; ++i) {
		if ( ! bitRead(status, i)) { // want action
			if (i & 1) {	// write buffer, read it
				if (fifo[i].notFull()) fifo[i].push(ReadSPI(i));
			} else {	// read buffer, write it
				if (fifo[i].notEmpty()) WriteSPI(i, fifo[i].get());
			};
		};
	};
}	// }}}
//
void setup() {		// {{{ Setup
	// {{{ Pins
	// I2C
	// pinMode( P_I2C_SDA,	OUTPUT);	digitalWrite( P_I2C_SDA,	HIGH );	// P_I2C_SDA
	// pinMode( P_I2C_CLK,	OUTPUT);	digitalWrite( P_I2C_CLK,	HIGH );	// P_I2C_CLK
	pinMode( P_SR_reg0,	OUTPUT);	digitalWrite( P_SR_reg0,	HIGH );	// P_SR_reg0 not ready yet = StatusOff
	// Free to use
	// pinMode( P_D5,	OUTPUT);	digitalWrite( P_D5,	HIGH );	// P_D5
	// pinMode( P_D6,	OUTPUT);	digitalWrite( P_D6,	HIGH );	// P_D6
	// pinMode( P_D7,	OUTPUT);	digitalWrite( P_D7,	HIGH );	// P_D7
	// pinMode( P_D8,	OUTPUT);	digitalWrite( P_D8,	HIGH );	// P_D8
	// pinMode( P_D9,	OUTPUT);	digitalWrite( P_D9,	HIGH );	// P_D9
	// SPI
	pinMode( P_Demux2,	OUTPUT);	digitalWrite( P_Demux2,		HIGH );	// P_Demux2
	pinMode( P_SerW,	OUTPUT);	digitalWrite( P_SerW,		HIGH );	// P_SerW
	pinMode( P_SerRxx,	INPUT );	// digitalWrite( P_SerRxx,	HIGH );	// P_SerRxx
	pinMode( P_clock,	OUTPUT);	digitalWrite( P_clock,		HIGH );	// P_clock
	pinMode( P_Sel_Enable,	OUTPUT);	digitalWrite( P_Sel_Enable,	HIGH );	// P_Sel_Enable
	pinMode( P_Sel_0,	OUTPUT);	digitalWrite( P_Sel_0,		HIGH );	// P_Sel_0
	pinMode( P_Sel_1,	OUTPUT);	digitalWrite( P_Sel_1,		HIGH );	// P_Sel_1
	pinMode( P_Sel_2,	OUTPUT);	digitalWrite( P_Sel_2,		HIGH );	// P_Sel_2
	// SS - Slave Select must be OUTPUT
	pinMode( P_Ser_SS,	OUTPUT);	digitalWrite( P_Ser_SS,		HIGH );	// P_Ser_SS
	// Arduino LEDs
	pinMode( P_RX,		OUTPUT);	digitalWrite( P_RX,		HIGH );	// P_RX - VCC - HIGH = Off - RxOff
	pinMode( P_TX,		OUTPUT);	digitalWrite( P_TX,		HIGH );	// P_TX - VCC - HIGH = Off - TxOff
	// }}}
	// {{{ SPI
	SPI.beginTransaction (SPISettings (F_CPU / 2, MSBFIRST, SPI_MODE0));	// pokud by se to lišilo pro různé SPI, tak to dát do SelectSPI + do DeSelectSPI dát SPI.endTransaction()
	// }}}
	// {{{ I2C
	Wire.begin();		// join i2c bus (address optional for master)
	// }}}
	// {{{ Serial to PC
	Serial.begin(SERIAL_SPEED);
	Serial.println(F(VERZE));
	Serial.flush();
	// }}}
	RxOff;TxOff;	// set the led off after Serial.xxx
	ReadSPI(0);	// {{{ read WriteBuffers to give them to 6809
	ReadSPI(1);
	ReadSPI(3);
	ReadSPI(5);
	ReadSPI(7);	// }}}
	StatusOn; 	// now is the card initialized and ready
//	setDS3231time(0,3,1,4,16,8,23);
//	ATxx_write_uint16_t(AT24C32_I2C_ADDRESS,0,4);
//	ATxx_write_uint16_t(AT24C32_I2C_ADDRESS,4,0xDEAD);
//	ATxx_write_uint16_t(AT24C32_I2C_ADDRESS,0,2);

/*	{{{
	Wire.beginTransmission(AT24C32_I2C_ADDRESS);
	int index = 16;
	Wire.write(index >> 8);
	Wire.write(index & 0xFF);
	for (int i=0; i<64; i++) Wire.write(i & 0xFF);
	Wire.endTransmission();
	delay(10);		// FIXME
}}} */
/*	{{{ 
	Wire.beginTransmission(AT24C32_I2C_ADDRESS);
	int index = 0;
	Wire.write(index >> 8);
	Wire.write(index & 0xFF);
	for (int i=0; i<64; i++) Wire.write(i & 0xFF);
	Wire.endTransmission();
	delay(10);		// FIXME
 }}} */
}	// }}}
//
unsigned long last_mill;
bool phase;	// 0 read; 1 write
uint16_t cnt1,cnt2;

void loop() {	// {{{
	phase=false;
	cnt1=0;
	cnt2=0;
	last_mill=millis();
	StatusOn;	// -> 0
	while((millis()-last_mill)<=SpeedInterval){
		ReadSPI(7);
		++cnt1;
	};
	last_mill=millis();
	phase=true;
	cnt2=0;
	StatusOff;	// -> 1
	while((millis()-last_mill)<=SpeedInterval){
		WriteSPI(6,0);
		++cnt2;
	};
	StatusOn;	// -> 0
	Serial.print("SI: ");
	Serial.print(SpeedInterval);
	Serial.print("; R: ");
	Serial.print(cnt1);
	Serial.print("; W: ");
	Serial.print(cnt2);
	Serial.print("; @: ");
	Serial.print(last_mill);
	Serial.println();
}	// }}}
